from hspylib.core.enum.enumeration import Enumeration


class HttpMethod(Enumeration):
    OPTIONS = 'menu_options'
    HEAD = 'head'
    GET = 'get'
    POST = 'post'
    PUT = 'put'
    PATCH = 'patch'
    DELETE = 'delete'
